# Copyright (c) 2012 Linaro Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mocker import MockerTestCase

from lava_test.core.providers import RegistryProvider


class RegistryProviderTests(MockerTestCase):

    # Helper values used in test methods
    _code = 1
    _reason = "reason"
    _test_id = 'test1'
    _test_id_list = ['test1', 'test2']
    _url = "http://example.org/test.json"

    def _prepare_provider_that_crashes_with_error(self, error):
        self.provider = self.mocker.patch(
            RegistryProvider({'entries': [self._url]}))
        self.expect(self.provider._fill_cache()).passthrough()
        self.expect(self.provider._load_remote_test(self._url)).throw(error)

    def _test_fill_cache_response(self):
        # Now onto testing:
        self.mocker.replay()
        # Call _fill_cache()
        self.provider._fill_cache()
        # the actual tests are validated as mocker expectation

    def test_fill_cache_IOError_handling_with_just_reason(self):
        # We want to test resilience to IOError that has a 'reason' attribute
        error = IOError()
        error.reason = self._reason
        self._prepare_provider_that_crashes_with_error(error)
        # Now this ought to log a few warning messages
        logging = self.mocker.replace("logging", passthrough=False)
        logging.warning(
            "Unable to load test definition from %r: %r",
            self._url, error)
        logging.warning("Error reason: %r", self._reason)
        self._test_fill_cache_response()

    def test_fill_cache_IOError_handling_with_just_code(self):
        # We want to test resilience to IOError that has a 'code' attribute
        error = IOError()
        error.code = self._code
        provider = self._prepare_provider_that_crashes_with_error(error)
        # Now this ought to log a few warning messages
        logging = self.mocker.replace("logging", passthrough=False)
        logging.warning(
            "Unable to load test definition from %r: %r",
            self._url, error)
        logging.warning("Error code: %r", self._code)
        self._test_fill_cache_response()

    def test_fill_cache_IOError_handling_without_anything(self):
        # We want to test resilience to IOError that has no attributes
        error = IOError()
        provider = self._prepare_provider_that_crashes_with_error(error)
        # Now this ought to log a few warning messages
        logging = self.mocker.replace("logging", passthrough=False)
        logging.warning(
            "Unable to load test definition from %r: %r",
            self._url, error)
        self._test_fill_cache_response()

    def test_fill_cache_ValueError_handling(self):
        # We want to test resilience to ValueError 
        error = ValueError()
        provider = self._prepare_provider_that_crashes_with_error(error)
        # Now this ought to log a few warning messages
        logging = self.mocker.replace("logging", passthrough=False)
        logging.warning(
            "Unable to load test definition from %r: %r",
            self._url, error)
        self._test_fill_cache_response()

    def test_iter_calls_fill_cache(self):
        provider = self.mocker.patch(RegistryProvider({}))
        self.expect(iter(provider)).passthrough()
        self.expect(provider._fill_cache())
        self.expect(provider._cache.iterkeys()).generate(self._test_id_list)
        self.mocker.replay()
        self.assertSequenceEqual(list(provider), self._test_id_list)

    def test_getitem_calls_fill_cache(self):
        provider = self.mocker.patch(RegistryProvider({}))
        self.expect(provider[self._test_id]).passthrough()
        self.expect(provider._fill_cache())
        self.expect(provider._cache[self._test_id])
        self.mocker.replay()
        provider[self._test_id]
